@@ -438,8 +438,8 @@ thumb_loader_get_path (ThumbLoader *tl)
 }
 
 
-void 
-thumb_loader_start (ThumbLoader *tl)
+static void
+thumb_loader_start__step2 (ThumbLoader *tl)
 {
 	ThumbLoaderPrivateData *priv;
 	char *cache_path = NULL;
@@ -454,18 +454,20 @@ thumb_loader_start (ThumbLoader *tl)
 		time_t  mtime;
 
 		mtime = get_file_mtime (priv->uri);
-		if (gnome_thumbnail_factory_has_valid_failed_thumbnail (priv->thumb_factory, 
-									priv->e_uri, 
-									mtime)) {
+
+		cache_path = gnome_thumbnail_factory_lookup (priv->thumb_factory, 
+							     priv->e_uri, 
+							     mtime);
+
+		if ((cache_path == NULL)
+		    && gnome_thumbnail_factory_has_valid_failed_thumbnail (priv->thumb_factory, 
+									   priv->e_uri, 
+									   mtime)) {
 			g_signal_emit (G_OBJECT (tl),
 				       thumb_loader_signals[THUMB_ERROR],
 				       0);
 			return;
 		}
-
-		cache_path = gnome_thumbnail_factory_lookup (priv->thumb_factory, 
-							     priv->e_uri, 
-							     mtime);
 	}
 
 	if (cache_path != NULL) {
@@ -495,6 +497,12 @@ thumb_loader_start (ThumbLoader *tl)
 	image_loader_start (priv->il);
 }
 
+void 
+thumb_loader_start (ThumbLoader *tl)
+{
+	thumb_loader_stop (tl, (DoneFunc)thumb_loader_start__step2, tl);
+}
+
 
 void 
 thumb_loader_stop (ThumbLoader *tl,
